from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import constr
from pydantic import Field

from _incydr_sdk.enums.watchlists import WatchlistType


class UpdateExcludedUsersRequest(BaseModel):
    userIds: Optional[List[str]] = Field(
        None,
        description="A list of user IDs to add or remove.",
        max_items=100,
    )


class UpdateIncludedDepartmentsRequest(BaseModel):
    departments: Optional[List[str]] = Field(
        None, description="A list of departments to add or remove."
    )


class UpdateIncludedDirectoryGroupsRequest(BaseModel):
    groupIds: Optional[List[str]] = Field(
        None, description="A list of group IDs to add or remove."
    )


class UpdateIncludedUsersRequest(BaseModel):
    userIds: Optional[List[str]] = Field(
        None,
        description="A list of user IDs to add or remove.",
        max_items=100,
    )
    watchlistId: Optional[str] = Field(
        None, description="A unique watchlist ID.", example="123"
    )


class CreateWatchlistRequest(BaseModel):
    description: Optional[constr(max_length=250)] = Field(
        None,
        description="The optional description of a custom watchlist.",
        example="List of users that fit a custom use case.",
    )
    title: Optional[constr(max_length=50)] = Field(
        None,
        description="The required title for a custom watchlist.",
        example="My Custom List",
    )
    watchlistType: WatchlistType

    class Config:
        use_enum_values = True


class ListWatchlistsRequest(BaseModel):
    page: int = 1
    pageSize: int = 100
    userId: Optional[str]


class UpdateWatchlistRequest(BaseModel):
    description: Optional[constr(max_length=250)] = Field(
        None,
        description="The updated description of a custom watchlist.",
        example="List of users that fit a custom use case.",
    )
    title: Optional[constr(max_length=50)] = Field(
        None,
        description="The updated title for a custom watchlist.",
        example="My Custom List",
    )



from __future__ import annotations

from datetime import datetime
from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import Field
from pydantic import validator

from _incydr_sdk.core.models import ResponseModel
from _incydr_sdk.enums import _Enum


class SortKeys(_Enum):
    """Possible keys to sort agents list results by."""

    NAME = "NAME"
    USER_ID = "USER_ID"
    AGENT_TYPE = "AGENT_TYPE"
    OS_HOSTNAME = "OS_HOSTNAME"
    LAST_CONNECTED = "LAST_CONNECTED"
    OS_NAME = "OS_NAME"


class AgentType(_Enum):
    """Possible types of agents."""

    CODE42AAT = "CODE42AAT"
    COMBINED = "COMBINED"
    CODE42 = "CODE42"


class Agent(ResponseModel):
    """
    A model representing an Incydr agent.

    **Fields**:

    * **agent_id**: `str` The globally unique ID (guid) for this agent.
    * **name**: `str` The editable name of the agent.
    * **user_id**: `str` The unique ID of the user the agent is assigned to.
    * **os_hostname**: `str` The hostname reported by the OS the agent is running on.
    * **os_name**: `str` The name of the OS the agent is running on.
    * **machine_id**: `str` Device machine ID.
    * **serial_number**: `str` Authenticated agent serial number.
    * **active**: `bool` If the agent status is active.
    * **agent_type**: [`AgentType`][agent-type] The type of agent.
    * **agent_health_issue_types: `List[str]` List of health issues with the agent. Health issue types include the following: `NOT_CONNECTING`, `NOT_SENDING_SECURITY_EVENTS`.
    * **app_version**: `str` The app version of the agent.
    * **product_version**: `str` The product version of the agent.
    * **last_connected**: `datetime` The time the agent last connected to a Code42 Authority server.
    * **external_reference**: `str` Editable reference information (useful for identifying an agent in external systems).
    * **creation_date**: `datetime` The time the agent was first registered.
    * **modification_date**: `datetime` The time the agent's database entry was last updated.
    """

    agent_id: Optional[str] = Field(alias="agentId")
    name: Optional[str]
    user_id: Optional[str] = Field(alias="userId")
    os_hostname: Optional[str] = Field(alias="osHostname")
    os_name: Optional[str] = Field(alias="osName")
    machine_id: Optional[str] = Field(alias="machineId")
    serial_number: Optional[str] = Field(alias="serialNumber")
    active: Optional[bool]
    agent_type: Optional[AgentType] = Field(alias="agentType")
    agent_health_issue_types: Optional[List[str]] = Field(alias="agentHealthIssueTypes")
    app_version: Optional[str] = Field(alias="appVersion")
    product_version: Optional[str] = Field(alias="productVersion")
    last_connected: Optional[datetime] = Field(alias="lastConnected")
    external_reference: Optional[str] = Field(alias="externalReference")
    creation_date: Optional[datetime] = Field(alias="creationDate")
    modification_date: Optional[datetime] = Field(alias="modificationDate")


class AgentsPage(ResponseModel):
    """
    A model representing a page of Agents.

    **Fields**:

    * **agents**: `List[[Agent][agent-model]]` The list of agents returned from the query.
    * **total_count**: `int` Total number of agents found in query results.
    * **page_size**: `int` The maximum number of agents returned in query results page.
    * **page_num**: `int` The current page number of the query result set.
    """

    agents: List[Agent]
    total_count: int = Field(alias="totalCount")
    page_size: int = Field(alias="pageSize")
    page_num: int = Field(alias="page")


class AgentUpdateRequest(BaseModel):
    name: str
    externalReference: str


class QueryAgentsRequest(BaseModel):
    active: Optional[bool]
    agentType: Optional[AgentType]
    agentHealthy: Optional[bool]
    anyOfAgentHealthIssueTypes: Optional[List[str]]
    srtKey: Optional[SortKeys]
    srtDir: Optional[str]
    pageSize: Optional[int]
    page: Optional[int]

    @validator("srtDir")
    def _validate(cls, value):  # noqa
        value = str(value).upper()
        assert value in ("ASC", "DESC")
        return value



from itertools import count

import requests

from ..exceptions import IncydrException
from .models import Actor
from .models import ActorFamily
from .models import ActorsPage
from .models import QueryActorsRequest


class ActorsV1:
    """
    Client for `/v1/actors` endpoints.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.actors.v1.get_page()
    """

    def __init__(self, parent):
        self._parent = parent

    def get_page(
        self,
        active: bool = None,
        name_starts_with: str = None,
        name_ends_with: str = None,
        page_num: int = 1,
        page_size: int = 500,
        prefer_parent: bool = False,
    ) -> ActorsPage:
        """
        Get a page of actors.

        Filter results by passing the appropriate parameters:

        **Parameters**:

        * **active**: `bool` - Find actors with the given active state. When `True`, returns only active actors. When `False` returns only inactive actors. Defaults to `None`, returning both.
        * **name_starts_with**: `str` - Find actors whose name (e.g. username/email) starts with this text, ignoring case.
        * **name_ends_with**: `str` - Find actors whose name (e.g. username/email) ends with this text, ignoring case.
        * **page_num**: `int` - Page number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return per page. Must be between 1 and 500. Defaults to 500.
        * **prefer_parent**: `bool` = Returns an actor's parent when applicable. Returns an actor themselves if they have no parent.

        **Returns**: An [`Actor`][actor-model] object representing the actor.
        """
        request_path = "/v1/actors/actor/search"

        if prefer_parent:
            request_path += "/parent"

        data = QueryActorsRequest(
            nameStartsWith=name_starts_with,
            nameEndsWith=name_ends_with,
            active=active,
            pageSize=page_size,
            page=page_num,
        )

        response = self._parent.session.get(request_path, params=data.dict())
        return ActorsPage.parse_response(response)

    def iter_all(
        self,
        active: bool = None,
        name_starts_with: str = None,
        name_ends_with: str = None,
        page_size: int = 500,
        prefer_parent: bool = False,
    ) -> ActorsPage:
        """
        Iterate over all actors.

        Accepts the same parameters as `.get_page()` excepting `page_num`.

        **Returns**: A generator yielding individual [`Actor`][actor-model] objects.
        """
        for page_num in count(1):
            page = self.get_page(
                active=active,
                name_starts_with=name_starts_with,
                name_ends_with=name_ends_with,
                page_size=page_size,
                page_num=page_num,
                prefer_parent=prefer_parent,
            )
            yield from page.actors
            if len(page.actors) < page_size:
                break

    def get_actor_by_id(self, actor_id: str, prefer_parent: bool = False) -> Actor:
        """
        Get an actor by their actor ID.

        **Parameters**:

        * **actor_id**: `str` (required) - Unique ID for the actor.
        * **prefer_parent**: `str` - Returns an actor's parent when applicable. Returns an actor themselves if they have no parent.

        **Returns**: An [`Actor`][actor-model] object representing the actor.
        """
        request_path = f"/v1/actors/actor/id/{actor_id}"
        if prefer_parent:
            request_path += "/parent"

        try:
            response = self._parent.session.get(request_path)
            return Actor.parse_response(response)
        except requests.HTTPError as err:
            if err.response.status_code == 404:
                raise ActorNotFoundError

    def get_actor_by_name(self, name: str, prefer_parent: bool = False) -> Actor:
        """
        Get an actor by their name.

        **Parameters**:

        * **name**: `str` (required) - The actor name.
        * **prefer_parent**: `str` - Returns an actor's parent when applicable. Returns an actor themselves if they have no parent.

        **Returns**: An [`Actor`][actor-model] object representing the actor.
        """
        if prefer_parent:
            try:
                response = self._parent.session.get(
                    f"/v1/actors/actor/name/{name}/parent"
                )
                return Actor.parse_response(response)
            except requests.HTTPError as err:
                if err.response.status_code == 404:
                    raise ActorNotFoundError
        else:
            matches = self.get_page(name_starts_with=name)
            for actor in matches.actors:
                if actor.name == name:
                    return actor
            raise ActorNotFoundError(name)

    def get_family_by_member_id(self, actor_id: str) -> ActorFamily:
        """
        Get an actor family from the actor ID of any family member.

        **Parameters**:

        * **actor_id**: `str` (required) - Unique ID for an actor.

        **Returns**: An [`Actor Family`][actor-family-model] object representing the family of actors.
        """
        try:
            response = self._parent.session.get(
                f"/v1/actors/actor/id/{actor_id}/family"
            )
            return ActorFamily.parse_response(response)
        except requests.HTTPError as err:
            if err.response.status_code == 404:
                raise ActorNotFoundError

    def get_family_by_member_name(self, name: str) -> ActorFamily:
        """
        Get an actor family from the name of any family member.

        **Parameters**:

        * **name**: `str` (required) - Name of an actor.

        **Returns**: An [`ActorFamily`][actor-family-model] object representing the family of actors.
        """
        try:
            response = self._parent.session.get(f"/v1/actors/actor/name/{name}/family")
            return ActorFamily.parse_response(response)
        except requests.HTTPError as err:
            if err.response.status_code == 404:
                raise ActorNotFoundError(name)


class ActorsClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = ActorsV1(self._parent)
        return self._v1


class ActorNotFoundError(IncydrException):
    """Raised when a watchlist with the matching type or title is not found."""

    def __init__(self, identifier=None):
        self.identifier = identifier
        self.message = (
            (f"Actor Not Found Error: No results for actors matching '{identifier}'.")
            if identifier
            else "Actor Not Found Error: No results for specified actor(s)."
        )
        super().__init__(self.message)



from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import Field

from _incydr_sdk.core.models import ResponseModel


class QueryActorsRequest(BaseModel):
    nameStartsWith: Optional[str]
    nameEndsWith: Optional[str]
    active: Optional[bool]
    pageSize: Optional[int]
    page: Optional[int]


class Actor(ResponseModel):
    """
    A model representing an Actor.

    **Fields**:

    * **active**: `bool` Whether or not the actor is active.
    * **actor_id**: `str` The unique ID of the actor.
    * **alternate_names**: `List[str]` A list of other names the actor may have.
    * **country**: `str` The actor's country.
    * **department**: `str` The actor's department.
    * **division**: `str` The actor's division.
    * **employee_type**: `str` The actor's employment type, such as if they're a contractor.
    * **first_name**: `str` The first (given) name of the actor.
    * **in_scope**: `str` The actor's scope state. An actor is considered "in scope" if their activity is monitored in at least one data source.
    * **last_name**: `str` The last (family) name of the actor.
    * **locality**: `str` - The actor's locality (city).
    * **manager_actor_id**: `str` The actor ID of the actor's manager.
    * **name**: `str` The actor's (eg. full username/email) name.
    * **parentActorId**: `str` The actor ID of this actor's parent actor. (if the actor has a parent).
    * **region**: `str` The actor's region (state).
    * **title**: `str` The actor's job title.
    """

    active: Optional[bool] = Field(
        None, description="Whether or not the actor is active.", example=True
    )
    actor_id: Optional[str] = Field(
        None, alias="actorId", description="The actor ID.", example="112233445566"
    )
    alternate_names: Optional[List[str]] = Field(
        None,
        alias="alternateNames",
        description="A list of other names the actor may have.",
        example=["johnsmith@gmail.com", "john-smith@company.com"],
    )
    country: Optional[str] = Field(
        None, description="The actor's country", example="United States"
    )
    department: Optional[str] = Field(
        None, description="The actor's department", example="Product"
    )
    division: Optional[str] = Field(
        None, description="The actor's division", example="Engineering"
    )
    employee_type: Optional[str] = Field(
        None,
        alias="employeeType",
        description="The actor's employment, such as if they're a contractor.",
        example="full-time",
    )
    first_name: Optional[str] = Field(
        None,
        alias="firstName",
        description="The first name of the actor.",
        example="Smith",
    )
    in_scope: Optional[bool] = Field(
        None,
        alias="inScope",
        description="The actor's scope state. An actor is considered 'in scope' if their activity is monitored in at least one data source.",
        example=True,
    )
    last_name: Optional[str] = Field(
        None,
        alias="lastName",
        description="The last name of the actor.",
        example="John",
    )
    locality: Optional[str] = Field(
        None, description="The actor's locality (city).", example="Minneapolis"
    )
    manager_actor_id: Optional[str] = Field(
        None,
        alias="managerActorId",
        description="The actor ID of the actor's manager",
        example="9988776655",
    )
    name: Optional[str] = Field(
        None,
        description="The actor's (eg. full username/email) name.",
        example="john.smith@gmail.com",
    )
    parent_actor_id: Optional[str] = Field(
        None,
        alias="parentActorId",
        description="The actor ID of this actor's parent actor. (if the actor has a parent).",
        example="442244224422",
    )
    region: Optional[str] = Field(
        None, description="The actor's region.", example="Minnesota"
    )
    title: Optional[str] = Field(
        None, description="The actor's job title", example="Software Engineer"
    )


class ActorFamily(ResponseModel):
    """
    A model representing an actor family.

    An actor family consists of one or more child actors adopted by a single parent. If an actor is not associated with
    any other actors, the family consists of only the single actor.

    **Fields**:

    * **children**: `List[[Actor][actor-model]]` The list of child actors associated with the parent.
    * **parent**: `[Actor][actor-model]` The parent actor of the family.
    """

    children: List[Actor] = Field(description="The child actors in the family.")
    parent: Actor = Field(description="The parent actor of the family.")


class ActorsPage(ResponseModel):
    """
    A model representing a page of Actors

    **Fields**:

    * **actors**: `List[Actor]` The list of actors returned from the query.
    """

    actors: List[Actor]
